@@ -1,17 +1,17 @@
-/* 
+/*
  * This file is part of SharedArray.
  * Copyright (C) 2014-2017 Mathieu Mirmont <mat@parad0x.org>
- * 
+ *
  * SharedArray is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation, either version 2 of the License, or
  * (at your option) any later version.
- * 
+ *
  * SharedArray is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
  * along with SharedArray.  If not, see <http://www.gnu.org/licenses/>.
  */
@@ -35,6 +35,7 @@ struct array_meta {
 	char	magic[16];
 	size_t	size;
 	int	typenum;
+	int	itemsize;
 	int	ndims;
 	npy_intp dims[NPY_MAXDIMS];
 } __attribute__ ((packed));
@@ -45,7 +46,7 @@ extern PyTypeObject PyArrayDescObject_Type;
 
 /* Main functions */
 extern PyObject *shared_array_create(PyObject *self, PyObject *args, PyObject *kwds);
-extern PyObject *shared_array_attach(PyObject *self, PyObject *args);
+extern PyObject *shared_array_attach(PyObject *self, PyObject *args, PyObject *kwds);
 extern PyObject *shared_array_delete(PyObject *self, PyObject *args);
 
 /* SHM list function */
@@ -53,13 +54,13 @@ extern PyObject *shared_array_delete(PyObject *self, PyObject *args);
 extern PyObject *shared_array_list(PyObject *self, PyObject *args);
 #endif
 
-/* Memory locking functions */
-extern PyObject *shared_array_msync(PyObject *self, PyObject *args, PyObject *kwds);
-extern PyObject *shared_array_mlock(PyObject *self, PyObject *args, PyObject *kwds);
-extern PyObject *shared_array_munlock(PyObject *self, PyObject *args, PyObject *kwds);
-
 /* Support functions */
 extern int open_file(const char *name, int flags, mode_t mode);
 extern int unlink_file(const char *name);
 
+/* Numpy compability */
+#if NPY_ABI_VERSION < 0x02000000
+  #define PyDataType_ELSIZE(descr) ((descr)->elsize)
+#endif
+
 #endif /* !__SHARED_ARRAY_H__ */
