@@ -1,17 +1,17 @@
-/* 
+/*
  * This file is part of SharedArray.
  * Copyright (C) 2014-2017 Mathieu Mirmont <mat@parad0x.org>
- * 
+ *
  * SharedArray is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation, either version 2 of the License, or
  * (at your option) any later version.
- * 
+ *
  * SharedArray is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
  * along with SharedArray.  If not, see <http://www.gnu.org/licenses/>.
  */
@@ -21,6 +21,7 @@
 #define NO_IMPORT_ARRAY
 
 #include <Python.h>
+#include <sys/types.h>
 #include <sys/mman.h>
 #include <sys/stat.h>
 #include <fcntl.h>
@@ -32,39 +33,44 @@
  */
 static PyObject *do_delete(const char *name)
 {
-	struct array_meta meta;
+	struct array_meta *meta;
 	int fd;
-	int size;
+	struct stat file_info;
+	size_t map_size;
+	void *map_addr;
 
 	/* Open the file */
 	if ((fd = open_file(name, O_RDWR, 0)) < 0)
 		return PyErr_SetFromErrnoWithFilename(PyExc_OSError, name);
 
-	/* Seek to the meta data location */
-	if (lseek(fd, -sizeof (meta), SEEK_END) < 0) {
+	/* Find the file size */
+	if (fstat(fd, &file_info) < 0) {
 		close(fd);
 		return PyErr_SetFromErrnoWithFilename(PyExc_OSError, name);
 	}
 
-	/* Read the meta data structure */
-	size = read(fd, &meta, sizeof (meta));
-	close(fd);
+	/* Ignore short files */
+	if (file_info.st_size < (off_t) sizeof (*meta)) {
+		close(fd);
+		PyErr_SetString(PyExc_IOError, "No SharedArray at this address");
+		return NULL;
+	}
+	map_size = file_info.st_size;
 
-	/* Catch read errors */
-	if (size <= 0)
+	/* Map the whole file into memory */
+	map_addr = mmap(NULL, map_size, PROT_READ, MAP_SHARED, fd, 0);
+	close(fd);
+	if (map_addr == MAP_FAILED)
 		return PyErr_SetFromErrnoWithFilename(PyExc_OSError, name);
 
-	/* Catch short reads */
-	if (size != sizeof (meta)) {
-		PyErr_SetString(PyExc_IOError, "No SharedArray at this address");
-		return NULL;
-	}
-
 	/* Check the meta data */
-	if (strncmp(meta.magic, SHARED_ARRAY_MAGIC, sizeof (meta.magic))) {
+	meta = (struct array_meta *) (map_addr + (map_size - sizeof (*meta)));
+	if (strncmp(meta->magic, SHARED_ARRAY_MAGIC, sizeof (meta->magic))) {
+		munmap(map_addr, map_size);
 		PyErr_SetString(PyExc_IOError, "No SharedArray at this address");
 		return NULL;
 	}
+	munmap(map_addr, map_size);
 
 	/* Unlink the file */
 	if (unlink_file(name) < 0)
